-- xUtils 4.0.1 by Everglow
-- current  functionality ( to disable a function change it to false):
-------------------------------------------------------------------------
	-- increases the camera zoom-out distance to the maximum
		local maxzoom = true;
-------------------------------------------------------------------------
	-- restores old Ready Check sound
		local readycheck = true;
-------------------------------------------------------------------------
	-- closes the edit box if you enter combat while typing in chat (default chat frame only)
		local smartedit = true;
-------------------------------------------------------------------------
	-- Disables the Battle.net Toast popup (always or only in combat)
		local notoast = true;        -- do not set both of these to true
		local notoastcombat = false; -- set only one of these to true or both to false
-------------------------------------------------------------------------
	-- autoselect most valuable quest reward
		local autoselectreward = true;
-------------------------------------------------------------------------
	-- moves the edit boxes to the top of the chat frames
		local topedit = true;
-------------------------------------------------------------------------
	-- sets your reputation bar to the current faction whenever you gain rep
		local smartrep = true;
-------------------------------------------------------------------------
	-- sets whether garbage collection is implemented when you die or change zones
	-- also can report how much memory is used by lua, and the growth rate since the last collection
		local memcycle = true;
		-- parameters
			local INTVL = 180; -- the minimum interval between updates in seconds (default is 3 minutes)
			local report = false; -- change this to true if you want a memory report (default is false)
			local lastcycle, lastmem = 0,0;
-------------------------------------------------------------------------
	-- colours friends list with class colours
		local colouredFL = true;
-------------------------------------------------------------------------
		
function xMsg(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end


-- local functions
function xUtils_onLoad(frame)
	if maxzoom then 
		SetCVar("CameraDistanceMax", 50);
--		SetCVar("CameraDistanceMoveSpeed", 10);
	end
	if smartedit or notoastcombat then
		frame:RegisterEvent("PLAYER_REGEN_DISABLED");
	end
	if notoastcombat then
		frame:RegisterEvent("PLAYER_REGEN_ENABLED");
	end
	if notoast then
		BNet_DisableToasts();
	end
	if autoselectreward then
		frame:RegisterEvent("QUEST_COMPLETE");
	end
	if readycheck then
		frame:RegisterEvent("READY_CHECK");
	end
	if smartrep  then
		frame:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE");
	end
	if memcycle  then
		frame:RegisterEvent("PLAYER_DEAD");
		frame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	end
	if topedit then
		for i = 1, NUM_CHAT_WINDOWS do
			local eb = _G[("ChatFrame%dEditBox"):format(i)]
			eb:ClearAllPoints()
			eb:SetPoint("BOTTOMLEFT",  ("ChatFrame%d"):format(i), "TOPLEFT",  -3, 20)
			eb:SetPoint("BOTTOMRIGHT", ("ChatFrame%d"):format(i), "TOPRIGHT", 3, 20)
		end
	end
end
	
function xUtils_OnEvent(frame, event, faction, ...)
	if not faction then faction = " nil"; end

	if event == "READY_CHECK" then
		PlaySoundFile("Sound\\Interface\\levelup2.wav")

	elseif event == "PLAYER_REGEN_DISABLED" then --player enters combat
		if smartedit then ChatFrame1EditBox:Hide(); end
		if notoastcombat then BNet_DisableToasts(); end

	elseif event == "PLAYER_REGEN_ENABLED" then -- player leaves combat
		BNet_EnableToasts();

	elseif event == "CHAT_MSG_COMBAT_FACTION_CHANGE" then
		local s,newFaction,factionName,isHeader,curRepBar;
		s,_,newFaction = string.find(faction, "Reputation with (.+) increased by (%d+).");
		if(s and newFaction ~= GetWatchedFactionInfo()) then
			for i = 1, GetNumFactions() do
				factionName,_,_,_,_,_,_,_,isHeader = GetFactionInfo(i);
				if(isHeader == nil and factionName == newFaction) then
					SetWatchedFactionIndex(i);
					xMsg("xUtils: Reputation bar set to "..newFaction);
					return;
				end;
			end;			
		end;	

	elseif event == "QUEST_COMPLETE" then -- Quest turn in
		local choices, high, hLink, hID = GetNumQuestChoices(), 0
		if(choices > 1) then
			for i=1, choices do
				local link	= GetQuestItemLink("choice", i);
				local _, _, _, _, _, _, _, _, _, _, value = GetItemInfo(link);
				if(value >= high) then
					high  = value;
					hLink = link;
					hID   = i;
				end
			end
		end
		if(high > 0) then
			local choice = _G["QuestInfoItem"..hID];
			QuestInfoFrame.itemChoice = hID;
			QuestInfoItemHighlight:SetPoint("TOPLEFT", choice, "TOPLEFT", -8, 7);
			QuestInfoItemHighlight:Show();
		end
	
	elseif event == "PLAYER_DEAD" or event == "ZONE_CHANGED_NEW_AREA" then --time to recycle unused memory
		local newtime = time(); -- tonumber(date("%j%H%M%S"));
		local elapsed = difftime(newtime,lastcycle);
		if elapsed > INTVL then
			local before = collectgarbage("count");
			local growth = (before-lastmem)/elapsed;
			collectgarbage();
			local after =  collectgarbage("count");
			if report then xMsg("memory recycled by xUtils:\nmemory freed: |c00FFFF66"..tostring(math.floor((before-after)+.5)).." kb|r\ncurrent memory use: |c00FFFF66"..tostring(math.floor((after/100)+.5)/10).." MB|r\nmemory growth rate: |c00FFFF66"..tostring((math.floor((growth*10)+.5)/10)).." kb|r/second."); end
			lastcycle = newtime;
			lastmem=after;
		end
	end
end

local function ColourFriendsList()
	local listSize = GetNumFriends()
	if listSize > 0 then
		local offset = FauxScrollFrame_GetOffset(FriendsFrameFriendsScrollFrame);
		for i=1, listSize do
			local name,_,class,_,connected = GetFriendInfo(i)
			local done = true;
			local colour
			for k,v in pairs(LOCALIZED_CLASS_NAMES_MALE) do
				if class == v then
					class = k;
					done = true;
					break;
				end
			end
			if done == false then
				for k,v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
					if class == v
						then class = k;
						break;
					end
				end
			end
			colour = RAID_CLASS_COLORS[class]
			if connected then
				local nameStr = getglobal("FriendsFrameFriendButton"..(i-offset).."ButtonTextName");
				if nameStr then
					nameStr:SetTextColor(colour.r, colour.g, colour.b, 1);
				end
			end
		end
	end
end
if colouredFL then hooksecurefunc("FriendsList_Update", ColourFriendsList); end